(****************************************************************)
(*								*)
(*		Modula-2 Compiler Source Module			*)
(*								*)
(*		  Load-builder driver program			*)
(*								*)
(*     (c) copyright 1990 Faculty of Information Technology	*)
(*		Queensland University of Technology		*)
(*								*)
(*     Permission is granted to use, copy and change this	*)
(*     program as long as the copyright message is left intact	*)
(*								*)
(****************************************************************)

MODULE Build;

(*
 *  this program may be called under various names:
 *	"build"  -- the standard build program
 *	"bldprf" -- build a profiling version
 *  the actual calling name is passed as argv[0] to
 *  the load builder code build2.
 *  By looking at argv[0] the build program can determine
 *  determine its intended task.
 *)

  IMPORT UxFiles, StdError, StdStrings, IntStr;
  
  FROM SYSTEM IMPORT CAST;
  FROM Types  IMPORT Int32;
  FROM ProgArgs IMPORT 
        UNIXexit, ArgNumber, GetArg, VersionTime, EnvironString;
  FROM BuildArgs IMPORT 
	Arg2, Arg3, ArgBlock, ArgsOf, NewArgBlock, AppendArg;
  FROM UxProcesses IMPORT Execp, Exec, Wait, Fork, ProcessID;
  FROM GpFiles IMPORT GpFindLocal;

  CONST usage = "usage : build [-VSv] [-Ldir] name";
	line0 = "   or : bldprf [-VSv] [-Ldir] name";
        line1 = " name should not have the '.mod' extension, options are --";
        line2 = "	-v verbose: filenames are traced";
        line3 = "	-V Version number (and more verbose)";
        line4 = "	-D Debug: automatic stack unwind on crash";
        line5 = 
"	-S Script: no link, startup and script in 'modbase' and 'modbase.c'";
        line6 = 
"	-Ldirname - directory 'dirname' is searched for archive libraries";
        line8 = "	-g include information for the debugger";

  CONST greetStr = "BUILD (Linux) version of ";

  TYPE  NameString = ARRAY [0 .. 79] OF CHAR;
        LongString = ARRAY [0 .. 255] OF CHAR;

  VAR	tmpNam : NameString;	(* name of temporary file  *)
	modNam : NameString;	(* name of the base module *)
        bldNam : NameString;    (* name of sym/rfx file    *)
        exeNam : NameString;    (* name of build2/o2build2 to call *)
        blk    : ArgBlock;	(* arg. block for build2   *)
	pidStr : ARRAY [0 .. 15] OF CHAR;


  VAR   argString : LongString;
	result : Int32;		(* result of "wait" call   *)

  VAR   child  : INTEGER;	(* the child process ident *)

	persistent : BOOLEAN;	(* ==> do not delete files *)
	hasOptions : BOOLEAN;   (* ==> pass options to b2  *)
	hasLinkDir : BOOLEAN;   (* ==> pass linkdir to b2  *)
	hasFileNam : BOOLEAN;   (* ==> has a single file   *)
	ok : BOOLEAN;

  VAR	argc, argIx, optIx, index : CARDINAL;
	optCh   : CHAR;
        bldFile : UxFiles.File;

  VAR	str0,			(* arg[0]    *)
	argN,			(* curr-arg  *)
	dirStr,			(* -Ldirname *)
	optStr	: NameString;   (* options   *)



BEGIN
  hasFileNam := FALSE;		(* init name string *)

  hasOptions := FALSE;		(* init option str  *)
  persistent := FALSE;
  optStr := "-";
  optIx  := 1;

  hasLinkDir := FALSE;		(* init linker str  *)
  dirStr := "";

  ok := TRUE;			(* the sticky flag  *)
  argc := ArgNumber();
  IF argc >= 1 THEN GetArg(0,str0) END;
  argIx := 1;
  WHILE argIx < argc DO
    GetArg(argIx,argN);
    IF argN[0] = "-" THEN
      IF argN[1] = "L" THEN
	IF hasLinkDir THEN ok := FALSE END; (* only one allowed *)
	hasLinkDir := TRUE;
    	dirStr := argN;
      ELSE (* some other option *)
	hasOptions := TRUE;
        index := 1;
	optCh := argN[index];
	WHILE optCh <> "" DO
	  optStr[optIx] := optCh;
	  IF optCh = "S" THEN persistent := TRUE END;
	  INC(optIx); INC(index);
	  optCh := argN[index];
	END;
      END; (* else some other option *)
    ELSE (* must be the filename *)
      IF hasFileNam THEN ok := FALSE END; (* only one allowed *)
      hasFileNam := TRUE;
      modNam := argN;
    END; (* if arg is option *)
    INC(argIx);
  END; (* for all args do ... *)
  IF hasOptions THEN optStr[optIx] := "" END;
  IF NOT ok OR NOT hasFileNam THEN (* bad args ... *)
    StdError.WriteString(greetStr); 
    VersionTime(argString);
    StdError.WriteString(argString);
    StdError.WriteLn;
    StdError.WriteString(usage); StdError.WriteLn;
    StdError.WriteString(line0); StdError.WriteLn;
    StdError.WriteString(line1); StdError.WriteLn;
    StdError.WriteString(line2); StdError.WriteLn;
    StdError.WriteString(line3); StdError.WriteLn;
    StdError.WriteString(line4); StdError.WriteLn;
    StdError.WriteString(line5); StdError.WriteLn;
    StdError.WriteString(line6); StdError.WriteLn;
    StdError.WriteString(line8); StdError.WriteLn;
    UNIXexit(1);
  END;
  IF persistent THEN tmpNam := "modbase";
  ELSE
    tmpNam := "/tmp/bld";
    IntStr.Give(pidStr,ProcessID(),1,IntStr.left);
    StdStrings.Append(pidStr,tmpNam);
  END;
  (* ************************************************************ *)
  (*
        New code added to allow search for GPM or GPO build
        files.
        pms Jun 93 (from D.Corney)
  *)
  GpFindLocal (modNam,"rfx",bldNam,bldFile);
  IF bldFile = NIL THEN           (* try for oberon build *)
     GpFindLocal (modNam,"sym",bldNam,bldFile);
     IF bldFile = NIL THEN        (* not modula or oberon *)
        StdError.WriteString ("** Base file not found **");
        StdError.WriteLn;
        UNIXexit (1);
     ELSE                         (* build oberon file    *)
        exeNam := "o2build2";
     END;
  ELSE                            (* build modula file    *)
     exeNam := "build2";
  END;
  (* ************************************************************ 
                          End Modula/Oberon check
     ************************************************************ *)

  IF Fork() = 0 THEN (* must be in the child process *)
    IF argc = 2 THEN
      Execp(exeNam, Arg3(str0,tmpNam,bldNam));
    ELSE 
      NewArgBlock(blk,9);
      AppendArg(blk,str0);
      IF hasOptions THEN AppendArg(blk,optStr) END;
      IF hasLinkDir THEN AppendArg(blk,dirStr) END;
      AppendArg(blk,tmpNam);
      AppendArg(blk,bldNam);
      Execp(exeNam,ArgsOf(blk));
    END;
    StdError.WriteString("build: can't exec ");
    StdError.WriteString(exeNam);
    StdError.WriteLn;
    UNIXexit(1);
  ELSE (* must be in parent process here *)
    child := Wait(result);
    IF result <> 0 THEN UNIXexit(result);
    ELSIF NOT persistent THEN (* must fork again *)
      IF Fork() = 0 THEN 
	Exec("/bin/sh",Arg2("sh",tmpNam));
	StdError.WriteString("build: can't shell load script");
	StdError.WriteLn;
	UNIXexit(1);
      ELSE (* in parent here *)
	child := Wait(result);
      END;
    ELSE UNIXexit(0);
    END;
  END;
  IF NOT persistent THEN
    modNam := tmpNam;
    UxFiles.Delete(modNam,ok);	   (* delete /tmp/bldNNNN OR modbase   *)
    StdStrings.Append(".dcf",modNam);
    UxFiles.Delete(modNam,ok);		(* delete /tmp/bldNNNN.dcf *)
    modNam := tmpNam;
    StdStrings.Append(".s",modNam);
    UxFiles.Delete(modNam,ok);		(* delete /tmp/bldNNNN.s   *)
    StdStrings.Delete(tmpNam,0,5);
    StdStrings.Append(".o",tmpNam);
    UxFiles.Delete(tmpNam,ok);      (* delete bldNNNN.o OR modbase.obj *)
  END;
  UNIXexit(0);
END Build.
