MODULE DrawMaze;

(*
 *  This program is copyright (c) Raoul Gough 1985 - 1990
 *
 *  Permission is granted to QUT to distribute this program
 *  as a demonstration program with the Gardens Point Modula
 *  compiler system
 *
 *  This program may be copied and or modified by users 
 *  for non-commercial purposes provided that this copyright 
 *  notice remains in the text
 *
 *  All other uses require the written permission of the 
 *  copyright holder.
 *)

(* Revision record:

   30 Aug 90  John Hynd		Portable colours matching graphics.mod

*)

FROM Random IMPORT Random;
FROM InOut IMPORT Write, WriteString, ReadCard, Read;
FROM Graphics IMPORT DrawBlock, GraphWindow, CloseScreen, Flush;
FROM SYSTEM IMPORT CAST;
FROM UxProcesses IMPORT Sleep;

CONST
  windowxMax = 500;
  windowyMax = 400;
  xPels = 5;
  yPels = 5;
  xMax = windowxMax DIV xPels;
  yMax = windowyMax DIV yPels;
  probBase = 100;
  redColour = 32 (*fire:22*);
  greenColour = 48 (*32*);
  blueColour = 79 (*39*);
  cyanColour = 109 (*51*);

  mazeColour = blueColour;
  tryColour = redColour;
  noGoodColour = (* cyanColour? *) blueColour - 6 (*4*);

TYPE
  Screen = ARRAY [0..xMax+1] OF ARRAY [0..yMax+1] OF BOOLEAN;

VAR
  occupied : Screen;
  x1, x2, y1, y2, yc, DotColour, level,
  maxlevel, xEnd, yEnd, turnProb,
  branchProb, branchNum,tx,ty,r         : CARDINAL;
  tempxDir, tempyDir : INTEGER;
  done:BOOLEAN;
  tempCh : CHAR;

  PROCEDURE DrawDot (x,y:CARDINAL);

  VAR
    yc : CARDINAL;
 
  BEGIN (* DrawDot *)
    DrawBlock (x * xPels - xPels, y * yPels - yPels, xPels, yPels, DotColour);
  END DrawDot;

  PROCEDURE Maze (x,y:CARDINAL; xDir,yDir:INTEGER);

    PROCEDURE Clear (x,y:CARDINAL; xDir,yDir:INTEGER) : BOOLEAN;
     
      PROCEDURE VertOk () :BOOLEAN;
      
      BEGIN (* VertOk *)
        IF occupied [x,CAST (INTEGER, y)+yDir] THEN
          RETURN FALSE
        ELSIF occupied [x-1,y] THEN
          RETURN FALSE
        ELSIF occupied [x+1,y] THEN
          RETURN FALSE
        ELSE
          RETURN TRUE;
        END;
      END VertOk;

      PROCEDURE HorizOk () : BOOLEAN;
      
      BEGIN (* HorizOk *)
        IF occupied [CAST (INTEGER, x)+xDir,y] THEN
          RETURN FALSE
        ELSIF occupied [x,y-1] THEN
          RETURN FALSE
        ELSIF occupied [x,y+1] THEN
          RETURN FALSE
        ELSE
          RETURN TRUE;
        END;
      END HorizOk;
   
    BEGIN (* Clear *)
      IF xDir = 0 THEN (* Must be vertical *)
        RETURN VertOk ();
      ELSE
        RETURN HorizOk ();
      END; (* IF ELSE *)
    END Clear;

    PROCEDURE GetRandomDirect (VAR tempxDir,tempyDir:INTEGER);

    BEGIN (* GetRandomDirect *)
      IF ODD (Random (10000)) THEN
        tempxDir:=yDir;
        tempyDir:=xDir;
      ELSE
        tempxDir:=-yDir;
        tempyDir:=-xDir;
      END;
    END GetRandomDirect;

    PROCEDURE TryBranch (VAR ok:BOOLEAN);

      PROCEDURE CheckTurn;

      BEGIN (* CheckTurn *)
        tx:=CAST (CARDINAL, CAST (INTEGER, x) + tempxDir);
        ty:=CAST (CARDINAL, CAST (INTEGER, y) + tempyDir);
        ok:=Clear (tx,ty,tempxDir,tempyDir);
      END CheckTurn;

    BEGIN (* TryBranch *)
      GetRandomDirect (tempxDir,tempyDir);
      CheckTurn;
      IF NOT ok OR (Random (100)<20) THEN
        tempxDir:=-tempxDir;
        tempyDir:=-tempyDir;
        CheckTurn;
      END;
    END TryBranch;
    
    PROCEDURE TryTurn (VAR ok:BOOLEAN);
    
      PROCEDURE CheckTurn;

      BEGIN (* CheckTurn *)
        tx:=CAST (CARDINAL, CAST (INTEGER, x) + tempxDir);
        ty:=CAST (CARDINAL, CAST (INTEGER, y) + tempyDir);
        ok:=Clear (tx,ty,tempxDir,tempyDir);
        IF ok THEN
          xDir:=tempxDir;
          yDir:=tempyDir;
        END;
      END CheckTurn;

    BEGIN (* TryTurn *)
      GetRandomDirect (tempxDir,tempyDir);
      CheckTurn;
      IF NOT ok THEN
        tempxDir:=-tempxDir;
        tempyDir:=-tempyDir;
        CheckTurn;
      END;
    END TryTurn;

  BEGIN (* Maze *)
    IF Clear (x,y,xDir,yDir) THEN
      INC (level);
      LOOP
        tx:=CAST (CARDINAL, CAST (INTEGER, x)+xDir);
        ty:=CAST (CARDINAL, CAST (INTEGER, y)+yDir);
        IF NOT Clear (tx,ty,xDir,yDir) THEN
          TryTurn (done);
          IF NOT done THEN EXIT END;
        ELSE
          x:=tx;
          y:=ty;
        END;
        DrawDot (x,y);
        occupied [x,y]:=TRUE;
        r:=Random (probBase);
        IF r>=branchNum THEN
          TryBranch (done);
          IF done THEN
            DrawDot (tx,ty);
            occupied [tx,ty]:=TRUE;
            Maze (tx,ty,tempxDir,tempyDir);
          END;
        ELSIF r<turnProb THEN
          TryTurn (done);
        END;
      END; (* LOOP *)
      IF level>maxlevel THEN
        maxlevel:=level;
        xEnd:=x;
        yEnd:=y;
      END;
      DEC (level);
    END; (* IF *)
  END Maze;

  PROCEDURE Solve;

  VAR
    x,y:CARDINAL;
    done:BOOLEAN;

    PROCEDURE CompSolve (x,y:CARDINAL);

    BEGIN (* CompSolve *)
      INC (level);
      done:=(occupied [x,y]);
      IF (x=2) AND (y=2) THEN
        done:=TRUE
      ELSIF done THEN
        DotColour:=tryColour;
        DrawDot (x,y);
        occupied [x,y]:=FALSE;
        CompSolve (x-1,y);
        IF NOT done THEN
          CompSolve (x,y-1);
          IF NOT done THEN
            CompSolve (x,y+1);
            IF NOT done THEN
              CompSolve (x+1,y);
              IF NOT done THEN
                DotColour:=noGoodColour;
                DrawDot (x,y);
                occupied [x,y]:=TRUE;
              END;
            END;
          END;
        END;
      END;
      DEC (level);
    END CompSolve;

  BEGIN (* Solve *)
    level:=0;
    CompSolve (xEnd,yEnd);
  END Solve;

  PROCEDURE Init;

  VAR
    x,y:CARDINAL;
  
  BEGIN (* Init *)
    FOR x:=1 TO xMax DO
      FOR y:=1 TO yMax DO
        occupied [x,y]:=FALSE;
      END;
    END;
    FOR x:=0 TO xMax DO
      occupied [x,0]:=TRUE;
      occupied [x,yMax+1]:=TRUE;
    END;
    FOR y:=1 TO yMax DO
      occupied [0,y]:=TRUE;
      occupied [xMax+1,y]:=TRUE;
    END;
    GraphWindow ("DrawMaze", windowxMax, windowyMax);
(*    WriteString ("Turn probability: ");
    ReadCard (turnProb);
    WriteString ("Branch probability: ");
    ReadCard (branchProb); *)
    branchProb := Random (probBase DIV 2) + 2;
    turnProb := Random (probBase DIV 2);
    branchNum:=probBase-branchProb;

    level:=0;
    maxlevel:=0;
    DotColour:=mazeColour;
   END Init;

   PROCEDURE Pause;

   VAR
     throw : CARDINAL;

   BEGIN (* Pause *)
     throw := Sleep (10);
   END Pause;

BEGIN (* DrawMaze *)
  REPEAT
    Init;
    Maze (1,2,1,0);

    DotColour := tryColour;
    DrawDot (xEnd, yEnd);
    Flush;
    Pause;

    Solve;
    Flush;
    Pause;
  UNTIL FALSE; (* Loop forever *)
END DrawMaze.
