MODULE Example6;

FROM Xglobal IMPORT int, unsignedInt, unsignedLong, Bool;
FROM Xlib IMPORT None, QueuedAlready, GXcopy, GXxor,
		DisplayPtr, Window, GC, XEvent, KeySym,
		XOpenDisplay, XDefaultScreen, XWhitePixel, XBlackPixel,
		XDisplayWidth, XDisplayHeight,
		XCreateSimpleWindow, XDefaultRootWindow, XStoreName,
		XCreateGC, XSetForeground, XSetFunction,
		XSelectInput,
		ExposureMask, PointerMotionMask, ButtonPressMask, KeyPressMask,
		Expose, MotionNotify, ButtonPress, KeyPress,
		XEventsQueued, XNextEvent, XCheckTypedEvent,
		XMapRaised,
		XDrawRectangle, XFillArc,
		XLookupString,
		XQueryPointer,
		XFlush,
		XDestroyWindow, XCloseDisplay;

FROM InOut IMPORT WriteString, WriteInt, WriteHex, WriteLn;

CONST
  degreeScale = 64;

VAR
mydisplay : DisplayPtr;
width, height : int;
mywindow : Window;
myscreen : int;
mygc : GC;
myforeground, mybackground, myXor : unsignedLong;
myevent : XEvent;
text : ARRAY [0..9] OF CHAR;
mykey : KeySym;
i : INTEGER;
rootwin, childwin : Window;
xRoot, yRoot, x, y, xOld, yOld : int;
buttons : unsignedInt;
discardBool : Bool;

BEGIN
WriteString ("Gpm X interface example 6:"); WriteLn;
WriteString ("Open window, enter event loop:"); WriteLn;
WriteString (" - any mouse button press: draw a filled circle;"); WriteLn;
WriteString (" meanwhile, follow pointer with rectangle;"); WriteLn;
WriteString (" - 'q' key: quit."); WriteLn;

(* Connect to the server: *)
	mydisplay := XOpenDisplay("");
	IF mydisplay = NIL THEN
		WriteString ("Cannot connect to server");
		WriteLn;
		HALT;
	END;
(* Get a screen: *)
	myscreen := XDefaultScreen(mydisplay);
(* Look up "black" and "white": *)
	mybackground := XWhitePixel(mydisplay, myscreen);
	myforeground := XBlackPixel(mydisplay, myscreen);
	IF myforeground <> 0 THEN
	  myXor := myforeground
	ELSE
	  myXor := 1;
	END;
(* Get width and height of root window: *)
	width := XDisplayWidth(mydisplay, myscreen);
	height := XDisplayHeight(mydisplay, myscreen);
(* Create a window based on screen size: *)
	mywindow := XCreateSimpleWindow(mydisplay,
		XDefaultRootWindow(mydisplay), 0, 0, width/3, height/3, 2,
		myforeground, mybackground);
(* Give the window manager a window name hint *)
        XStoreName(mydisplay, mywindow, "Gpm Example6");
(* Create a GC: *)
	mygc := XCreateGC(mydisplay, mywindow, 0, NIL);
(* Ask to receive Expose, MotionNotify, ButtonPress and KeyPress events: *)
	XSelectInput(mydisplay, mywindow, ExposureMask     +
					  PointerMotionMask +
					  ButtonPressMask  +
					  KeyPressMask      );
(* Pop this window up on the screen: *)
	XMapRaised(mydisplay, mywindow);
(* The event loop: *)
	LOOP
	    (* Get the next event in the event queue: *)
	    XNextEvent(mydisplay, myevent);
	    CASE myevent.type OF
	    | Expose :
		    (* Ignore Expose events *)
	    | MotionNotify :
		WHILE XCheckTypedEvent(mydisplay, MotionNotify, myevent) DO
		  (* nothing *) 
		  END;
		XSetFunction (mydisplay, mygc, GXxor);
		XSetForeground(mydisplay, mygc, myXor);
		XDrawRectangle(mydisplay, mywindow, mygc, x, y, 50, 50);
	        discardBool := XQueryPointer(mydisplay, mywindow,
		    rootwin, childwin, xRoot, yRoot, x, y, buttons);
		XDrawRectangle(mydisplay, mywindow, mygc, x, y, 50, 50);
	    | ButtonPress :
		(* User pressed a pointer button so draw circle *)
		XSetFunction (mydisplay, mygc, GXxor);
		XSetForeground(mydisplay, mygc, myXor);
		XDrawRectangle(mydisplay, mywindow, mygc, x, y, 50, 50);
		XSetFunction (mydisplay, mygc, GXcopy);
		XSetForeground(mydisplay, mygc, myforeground);
		XFillArc(mydisplay, mywindow, mygc,
		         myevent.xbutton.x, myevent.xbutton.y,
		         100, 75, 0, 270*degreeScale);
		XSetFunction (mydisplay, mygc, GXxor);
		XSetForeground(mydisplay, mygc, myXor);
		XDrawRectangle(mydisplay, mywindow, mygc, x, y, 50, 50);
	    | KeyPress :
		(* User pressed a key, see if it's a "q" *)
		i := XLookupString(myevent, text, 10, mykey, 0);
		IF (i=1) AND (text[0]='q') THEN
		    EXIT;
		END;
	    END;
	END;

(* Exit gracefully: *)
	XDestroyWindow(mydisplay, mywindow);
	XCloseDisplay(mydisplay);
END Example6.
