MODULE Quix;

(*
 *  This program is copyright (c) Raoul Gough 1985 - 1990
 *
 *  Permission is granted to QUT to distribute this program
 *  as a demonstration program with the Gardens Point Modula
 *  compiler system
 *
 *  This program may be copied and or modified by users 
 *  for non-commercial purposes provided that this copyright 
 *  notice remains in the text
 *
 *  All other uses require the written permission of the 
 *  copyright holder.
 *)

FROM Graphics IMPORT GraphWindow, HiResLine;
FROM Random IMPORT Random;
FROM InOut IMPORT WriteString, WriteCard, WriteLn, ReadCard;

CONST
  NumColours = 60;
  ZeroColour = 15;
  ColourStep = 5;
  MaxQuixes = 5;
  Length = 30;
  ScaleFactor = 10;
  xMax = 500;
  yMax = 300;

TYPE
  LinePoints = RECORD x1, y1, x2, y2 : INTEGER; END;

  QuixInfo = RECORD
               lines : ARRAY [0..Length] OF LinePoints;
               xMove1, yMove1, xMove2, yMove2 : INTEGER;
               slowness, colour, cursor, count1, count2 : CARDINAL;
             END; (* QuixInfo *)

VAR
  numQuixes : CARDINAL;
  quixes : ARRAY [1..MaxQuixes] OF QuixInfo;
  count : CARDINAL;

  PROCEDURE InitQuix (VAR q : QuixInfo);

  VAR
    count : CARDINAL;

  BEGIN (* InitQuix *)
    WITH q DO
      cursor := 0;
      count1 := 0; count2 := 0;
      slowness := Random (5) + 5; (*Random (15) + 5;*)
      FOR count := 0 TO Length DO
        WITH lines [count] DO
          x1 := xMax DIV 2; y1 := yMax DIV 2; x2 := x1; y2 := y1;
        END; (* WITH *)
      END; (* FOR *)
    END; (* WITH *)
  END InitQuix;

  PROCEDURE MoveQuix (VAR q : QuixInfo);

  VAR
    oldCursor : CARDINAL;

    PROCEDURE UpdateMove (VAR c : CARDINAL; VAR xM, yM : INTEGER;
                           xBias, yBias : INTEGER;
                           slowness : CARDINAL);

    BEGIN (* UpdateMove *)
      IF c > 0 THEN
        DEC (c);
      ELSE
        xM := INT (Random (ScaleFactor * 60 + 1)) - ScaleFactor * 30;
        yM := INT (Random (ScaleFactor * 40 + 1)) - ScaleFactor * 20;
        xM := (xM + xBias DIV 6) DIV INT (slowness);
        yM := (yM + yBias DIV 6) DIV INT (slowness);
        c := Random (20) + 15;
      END;
    END UpdateMove;

    PROCEDURE MovePoint (VAR x, y, xMove, yMove : INTEGER);

    BEGIN (* MovePoint *)
      x := x + xMove;
      y := y + yMove;
      IF (x < 0) OR (x > xMax * ScaleFactor - 1) THEN
        xMove := -xMove;
        x := x + xMove;
      END; (* IF *)
      IF (y < 0) OR (y > yMax * ScaleFactor - 1) THEN
        yMove := -yMove;
        y := y + yMove;
      END; (* IF *)
    END MovePoint;

    PROCEDURE DrawLine (l : LinePoints; c : CARDINAL);

    BEGIN (* DrawLine *)
      WITH l DO
        HiResLine (x1 DIV ScaleFactor, y1 DIV ScaleFactor,
              x2 DIV ScaleFactor, y2 DIV ScaleFactor, c);
      END; (* WITH *)
    END DrawLine;

  BEGIN (* MoveQuix *)
    WITH q DO
      oldCursor := cursor;
      cursor := (cursor + 1) MOD Length;
      DrawLine (lines [cursor], 0);
      lines [cursor] := lines [oldCursor];
      WITH lines [cursor] DO
        UpdateMove (count1, xMove1, yMove1, x2 - x1, y2 - y1, slowness);
        UpdateMove (count2, xMove2, yMove2, x1 - x2, y1 - y2, slowness);
        MovePoint (x1, y1, xMove1, yMove1);
        MovePoint (x2, y2, xMove2, yMove2);
      END; (* WITH *)
      DrawLine (lines [cursor], colour);
    END; (* WITH *)
  END MoveQuix;

BEGIN (* Quix *)
(*  REPEAT
    WriteString ("Number of quixes: ");
    ReadCard (numQuixes);
    IF numQuixes > MaxQuixes THEN
      WriteString ("Max is "); WriteCard (MaxQuixes, 1); WriteLn;
    END; (* IF *)
  UNTIL numQuixes <= MaxQuixes; *)
  numQuixes := 3;
  FOR count := 1 TO numQuixes DO
    InitQuix (quixes [count]);
    quixes[count].colour := (count * ColourStep) MOD (NumColours - 1)
				 + ZeroColour;
  END; (* FOR *)
  GraphWindow ("Quix", xMax, yMax);
  LOOP
    FOR count := 1 TO numQuixes DO MoveQuix (quixes[count]); END; (* FOR *)
  END; (* LOOP *)
END Quix.
