MODULE RealMand;

(*
 *  This program is copyright (c) Raoul Gough 1985 - 1990
 *
 *  Permission is granted to QUT to distribute this program
 *  as a demonstration program with the Gardens Point Modula
 *  compiler system
 *
 *  This program may be copied and or modified by users 
 *  for non-commercial purposes provided that this copyright 
 *  notice remains in the text
 *
 *  All other uses require the written permission of the 
 *  copyright holder.
 *)

FROM InOut IMPORT WriteLn, WriteInt, WriteCard;
FROM RealInOut IMPORT ReadReal, WriteReal;
FROM RealMath IMPORT round;
FROM Graphics IMPORT GetMouseClick, GraphWindow, CloseScreen, HiResDot, Flush,
PlaceGraphWindow,
			EventHappened;

CONST
  LeftMouseButton = 256;
  RightMouseButton = 1024;

  NumColours = 235;
  Skip = 1;		(* Use only every Skip'th colour *)
  ColourOffset = 13;	(* Avoid window manager and primary colours *)
  xMax = 1000;
  yMax = 800;

TYPE
  Complex = RECORD
              real, imag : REAL
            END;

VAR
  xc, yc, xmag, ymag, magnitudeLimit : REAL;
  squaresLimit : CARDINAL;

  opCount : CARDINAL;

  PROCEDURE MaxSquares (z : Complex) : CARDINAL;

  VAR
    count : CARDINAL;
    rsquared, isquared : REAL;
    temp : Complex;
    over : BOOLEAN;

  BEGIN (* MaxSquares *)
    temp := z;
    count := 1;
    WITH temp DO
      REPEAT
	rsquared := real * real;
	isquared := imag * imag;
	over := isquared + rsquared >= magnitudeLimit;
	opCount := opCount + 3;
	IF NOT over THEN
	  INC (count);
	  imag := 2.0 * real * imag + z.imag;
	  real := rsquared - isquared + z.real;
	  opCount := opCount + 3;
	END; (* IF *)
      UNTIL over OR (count = squaresLimit);
    END; (* WITH *)
    RETURN count;
  END MaxSquares;

  PROCEDURE DrawMandlebrot;

  VAR
    x, y : INTEGER;
    z : Complex;
    colour : CARDINAL;
    done : BOOLEAN;

  BEGIN (* DrawMandlebrot *)
    done := FALSE;
    y := 0;
    WHILE (y < yMax) AND NOT done DO
      x := 0;
      WHILE (x < xMax) AND NOT done DO
(*    FOR y := 0 TO yMax - 1 DO
      FOR x := 0 TO xMax - 1 DO *) 
        WITH z DO
	  real := FLOAT (x - xMax DIV 2) / xmag + xc;
	  imag := FLOAT (yMax DIV 2 - y) / ymag + yc;
	  colour := (MaxSquares (z) * Skip MOD NumColours) + ColourOffset;
	  HiResDot (x, y, colour);
        END; (* WITH  *)
	INC (x);
      END; (* FOR *)
      done := EventHappened ();
      INC (y);
    END; (* FOR *)
  END DrawMandlebrot;

  PROCEDURE GetNewParams;

  VAR
    oldScale, x0, y0, x1, y1, button : INTEGER;

  BEGIN (* GetNewParams *)
    GetMouseClick (x0, y0, button);
    GetMouseClick (x1, y1, button);
    xc := FLOAT (x0 - xMax DIV 2) / xmag + xc;
    yc := FLOAT (yMax DIV 2 - y0) / ymag + yc;
    xmag := xmag * FLOAT (xMax) / FLOAT (2 * ABS (x1 - x0));
    ymag := xmag;
    IF xmag > 32000.0 THEN
      squaresLimit := 1024;
    ELSIF xmag > 8000.0 THEN
      squaresLimit := 512;
    ELSIF xmag > 2000.0 THEN
      squaresLimit := 256;
    END;
    WriteCard (opCount, 1);
    WriteReal (xc, 10); WriteReal (yc, 10); WriteReal (xmag, 10); WriteLn;
    opCount := 0;
  END GetNewParams;

  PROCEDURE InitParams;

  BEGIN (* InitParams *)
    xc := 0.0;
    yc := 0.0;
    xmag := 100.0;
    ymag := xmag;
    magnitudeLimit := 4.0;
    squaresLimit := 100;

    opCount := 0;
  END InitParams;

BEGIN (* RealMand *)
  InitParams;
  GraphWindow ("Mandlebrot", xMax, yMax);
(*  PlaceGraphWindow ("Mandlebrot", 200, 200, 400, 400); *)
  REPEAT
    DrawMandlebrot;
    Flush;
    GetNewParams;
  UNTIL FALSE;
  CloseScreen;
END RealMand.
