MODULE SetColours;

FROM Xglobal IMPORT int, unsignedInt, unsignedLong, Bool;
FROM Xlib IMPORT
		Success,
		DisplayPtr, Window, GC, Font, XEvent, KeySym,
		XOpenDisplay, XDefaultScreen, XWhitePixel, XBlackPixel,
		XDisplayWidth, XDisplayHeight,
		XCreateSimpleWindow, XDefaultRootWindow, XStoreName,
		XCreateGC, XLoadFont, XSetFont,
		XSelectInput,
		ExposureMask, ButtonPressMask, KeyPressMask,
		Expose, ButtonPress, KeyPress,
		XNextEvent,
		XMapRaised,
		XDrawImageString,
		XLookupString,
		VisualScreenMask, VisualDepthMask,
		VisualPtr, VisualInfo, VisualInfoPtr, Colormap, XColor,
		XDisplayPlanes, XDefaultVisual, XGetVisualInfo, XFree,
		XDefaultColormap, XSetForeground, XFillRectangle,
		XQueryColor, XAllocColor, XAllocNamedColor,
		XDestroyWindow, XCloseDisplay;

FROM InOut IMPORT EOL, Read,
		Write, WriteString, WriteInt, WriteCard, WriteHex, WriteLn;
FROM ProgArgs IMPORT ArgNumber, GetArg;
FROM CharInfo IMPORT IsDigit;
FROM CardStr IMPORT Take, ConvResults;
FROM IntStr IMPORT Alignment, Give;
FROM StdStrings IMPORT Append, Length;
IMPORT Ascii;

TYPE
  IndexType = [1..80];
  String = ARRAY IndexType OF CHAR;
  CharSet = SET OF CHAR;

VAR
mydisplay : DisplayPtr;
width, height : int;
mywindow : Window;
myscreen : int;
mygc : GC;
myfont : Font;
myforeground, mybackground : unsignedLong;
myevent : XEvent;
text : String (*ARRAY [0..79] OF CHAR*);
argv1, moreText : ARRAY [0..20] OF CHAR;
mykey : KeySym;
i : INTEGER;
rootwin, childwin : Window;
x, y : int;
buttons : unsignedInt;
discardBool : Bool;

depth : int;
defVisual : VisualPtr;
vTemplate : VisualInfo;
visualListPtr : VisualInfoPtr;
visualsMatched : INTEGER;
currentVisual : CARDINAL;
cmap : Colormap;
maxColour, colourIndex : CARDINAL;
colour : XColor;
primary : CARDINAL;
colourDesc, primaryDesc : String;
length : [0..80];
index : IndexType;
screenColour, exactColour : XColor;
result : ConvResults;
rgbColour, goodColour : BOOLEAN;

PROCEDURE ExtractCard (rgbDesc : String; VAR primaryDesc : String;
		       VAR index : IndexType; VAR found : BOOLEAN);
VAR
  primIndex : IndexType;
BEGIN
  found := FALSE;
  WHILE NOT IsDigit(rgbDesc[index]) AND (rgbDesc[index] <> Ascii.nul) DO
    INC(index);
  END;
  primIndex := 1;
  WHILE IsDigit(rgbDesc[index]) DO
    found := TRUE;
    primaryDesc[primIndex] := rgbDesc[index];
    INC(primIndex);
    INC(index);
  END;
  primaryDesc[primIndex] := Ascii.nul;
END ExtractCard ;

PROCEDURE WriteColour (colour : XColor);

PROCEDURE WritePrimary (primary : CARDINAL);
BEGIN
  IF (primary MOD 256) = 0 THEN
    WriteCard (primary DIV 256, 3);
  ELSE
    WriteCard (primary, 5);
    WriteString (":16");
  END;
END WritePrimary;

BEGIN (* WriteColour *)
  WITH colour DO
    WriteCard (pixel, 5);
    WriteString (": r:");
    WritePrimary (red);
    WriteString (" g:");
    WritePrimary (green);
    WriteString (" b:");
    WritePrimary (blue);
  END;
END WriteColour ;

PROCEDURE GivePrimary (primary : CARDINAL);
BEGIN
  IF (primary MOD 256) = 0 THEN
    Give (moreText, primary DIV 256, 3, right);
  ELSE
    Give (moreText, primary, 5, right);
    Append (":16", moreText);
  END;
END GivePrimary;

BEGIN
WriteString ("Gpm X interface colour sampler:"); WriteLn;
WriteLn;

(* Connect to the server use $DISPLAY: *)
	mydisplay := XOpenDisplay("");
	IF mydisplay = NIL THEN
		WriteString ("Cannot connect to server");
		WriteLn;
		HALT;
	END;
(* Get a screen: *)
	myscreen := XDefaultScreen(mydisplay);
(* Look up "black" and "white": *)
	mybackground := XWhitePixel(mydisplay, myscreen);
	myforeground := XBlackPixel(mydisplay, myscreen);
(* Get width and height of root window: *)
	width := XDisplayWidth(mydisplay, myscreen);
	height := XDisplayHeight(mydisplay, myscreen);
(* Create a window based on screen size: *)
	mywindow := XCreateSimpleWindow(mydisplay,
		XDefaultRootWindow(mydisplay), 0, 0,
		width-100, height-100, 2,
		myforeground, mybackground);
(* Give the window manager a window name hint *)
        XStoreName(mydisplay, mywindow, "Gpm Colour sampler");
(* Create a GC: *)
	mygc := XCreateGC(mydisplay, mywindow, 0, NIL);

(* Get the display parameters: *)
	depth := XDisplayPlanes (mydisplay, myscreen);
	WriteString ("Depth = "); WriteCard (depth,1); WriteString (" planes");
	WriteLn;
        maxColour := 2; 
	FOR i := 1 TO depth-1 DO
          maxColour := maxColour*2;
	END;
        maxColour := maxColour-1;
	defVisual := XDefaultVisual (mydisplay, myscreen);
	vTemplate.screen := myscreen;
	vTemplate.depth := depth;
	visualListPtr := XGetVisualInfo (mydisplay,
		VisualScreenMask + VisualDepthMask, vTemplate, visualsMatched);
	FOR currentVisual := 1 TO ORD(visualsMatched) DO
	  WriteString ("Visual "); WriteCard (currentVisual,1);
	  WriteString (": ");
	  WITH visualListPtr^[currentVisual] DO
	    WriteString ("Visual class = ");
	    CASE class OF
	    | 0 : WriteString ("StaticGray");
	    | 1 : WriteString ("GrayScale");
	    | 2 : WriteString ("StaticColor");
	    | 3 : WriteString ("PseudoColor");
	    | 4 : WriteString ("TrueColor");
	    | 5 : WriteString ("DirectColor");
            ELSE
              WriteString ("! Unknown : ");
	      WriteCard (class,1);
	      WriteLn;
	    END;
	    IF visual = defVisual THEN
	      WriteString (" (default visual)");
	    END;
	  END;
	  WriteLn;
	END;
	XFree (visualListPtr);
	cmap := XDefaultColormap (mydisplay, myscreen);

(* Set up some of our own colours *)
	REPEAT
	  WriteString ("Colour to add to map? (empty to finish) : ");
	  length := 0;
	  REPEAT
	    length := length + 1;
	    Read(colourDesc[length]);
	  UNTIL colourDesc[length] = EOL;
	  IF length <> 1 THEN
	    colourDesc[length] := Ascii.nul;
	    (* Try to interpret as r g b cardinals *)
	    index := 1;
	    ExtractCard (colourDesc, primaryDesc, index, rgbColour);
	    (* Got a cardinal - assume r g b *)
	    IF rgbColour THEN
	      goodColour := FALSE;
	      WITH screenColour DO
	        Take (primaryDesc, primary, result);
	        IF result = allRight THEN
	          red := primary;
	          ExtractCard (colourDesc, primaryDesc, index, rgbColour);
	          Take (primaryDesc, primary, result);
	          IF result = allRight THEN
		    green := primary;
	            ExtractCard (colourDesc, primaryDesc, index, rgbColour);
	            Take (primaryDesc, primary, result);
	            IF result = allRight THEN
		      blue := primary;
	              goodColour :=  XAllocColor(mydisplay, cmap, screenColour)
			             = Success;
		    END;
		  END;
		END;
	      END;
	    ELSE (* not a cardinal - treat as colour name *)
	      rgbColour := FALSE;
	      goodColour := XAllocNamedColor(mydisplay, cmap, colourDesc,
			    screenColour, exactColour) = Success;
	    END;
	    IF goodColour THEN
	      WriteString ('Colour "');
	      WriteString (colourDesc);
	      WriteString ('":');
	      WriteLn;
	      WriteString ("Screen colour: ");
	      WriteColour (screenColour);
	      WriteLn;
	      IF NOT rgbColour THEN
	        WriteString ("Exact colour:  ");
	        WriteColour (exactColour);
	      END;
	    ELSE
	      WriteString ("Bad colour!");
	    END;
	    WriteLn;
	  END;
	UNTIL length = 1;

(* Ask to receive Expose and KeyPress events: *)
	XSelectInput(mydisplay, mywindow, ExposureMask     +
					  KeyPressMask      );
(* Pop this window up on the screen: *)
	XMapRaised(mydisplay, mywindow);

(* The event loop: *)
	LOOP
		(* Get the next event in the event queue: *)
		XNextEvent(mydisplay, myevent);
		CASE myevent.type OF
		| Expose :
			(* Ignore Expose events *)
(* Display all the colours *)
	x := 0; y := 0;
	FOR colourIndex := 0 TO maxColour DO
	  XSetForeground (mydisplay, mygc, colourIndex);
	  XFillRectangle (mydisplay, mywindow, mygc, x, y, 50, 50);
	  WITH colour DO
	    Give (text, colourIndex, 2, right);
	    Append (": r:", text);
	    pixel := colourIndex;
	    XQueryColor (mydisplay, cmap, colour);
	    GivePrimary (red);
	    Append (moreText, text);
	    Append (" g:", text);
	    GivePrimary (green);
	    Append (moreText, text);
	    Append (" b:", text);
	    GivePrimary (blue);
	    Append (moreText, text);
	    XDrawImageString(mydisplay, mywindow, mygc,
			     x+55, y+10, text, Length(text));
	  END;
	  IF y<(height - 150) THEN
	    INC (x,5);
	    INC (y,20);
	  ELSE
	    INC (x,50);
	    y := 0;
	  END;
	END;

		| KeyPress :
			(* User pressed a key, see if it's a "q" *)
			i := XLookupString(myevent, text, 10, mykey, 0);
			IF (i=1) AND (text[1]='q') THEN
				EXIT;
			END;
		END;
	END;
(* Exit gracefully: *)
	XDestroyWindow(mydisplay, mywindow);
	XCloseDisplay(mydisplay);
END SetColours.
