MODULE Snakes;

(*
 *  This program is copyright (c) Raoul Gough 1985 - 1990
 *
 *  Permission is granted to QUT to distribute this program
 *  as a demonstration program with the Gardens Point Modula
 *  compiler system
 *
 *  This program may be copied and or modified by users 
 *  for non-commercial purposes provided that this copyright 
 *  notice remains in the text
 *
 *  All other uses require the written permission of the 
 *  copyright holder.
 *)

FROM Graphics IMPORT GraphWindow, HiResDot, Flush;
FROM Random IMPORT Random;
FROM RealMath IMPORT cos, sin, round, PI;
FROM InOut IMPORT WriteString, WriteLn, ReadCard, WriteCard;

CONST
  xMax = 500;
  yMax = 300;
  NumColours = 60;
  ColourStep = 3;
  MaxSnakeLength=200;
  MaxSnakes=200;
  NumAngles=360;
  AngScale=8;
  ScaleAngles=AngScale * NumAngles;
  AngVariance=20;
  PointScale=32;
  RealPointScale=32.0;

TYPE
  Point = RECORD
            x,y:INTEGER;
          END; (* Point *)

  Snake = RECORD
            part : ARRAY [1..MaxSnakeLength] OF Point;
            angle,angMove,angMoveCount,xHead,yHead:INTEGER;
            snakeColour:CARDINAL;
          END; (* Snake *)

VAR
  snake:ARRAY [1..MaxSnakes] OF Snake;
  xmax,ymax,xmid,ymid:INTEGER;
  delay,snakelength,numsnakes:CARDINAL;
  sine,cosine:ARRAY [0..NumAngles-1] OF INTEGER;

  PROCEDURE GetParams;

  VAR
    tempInt:INTEGER;

  BEGIN (* GetParams *)
    REPEAT
      WriteString ('Number of snakes: ');
      ReadCard (numsnakes);
      IF numsnakes>MaxSnakes THEN
        WriteString ('Max snakes = ');
        WriteCard (MaxSnakes,1);
        WriteLn;
      END (* IF *)
    UNTIL (numsnakes <= MaxSnakes);
    REPEAT
      WriteString ('Snake length: ');
      ReadCard (snakelength);
      IF snakelength>MaxSnakeLength THEN
        WriteString ('Max length = ');
        WriteCard (MaxSnakeLength,1);
	WriteLn;
      END (* IF *)
    UNTIL (snakelength <= MaxSnakeLength);
    WriteString ("Delay: ");
    ReadCard (delay);
  END GetParams;

  PROCEDURE Init;

  VAR
    nextColour,c1,c2:CARDINAL;

  BEGIN (* Init *)
    GraphWindow ("snakes", xMax, yMax);
    xmid:=xMax DIV 2;
    ymid:=yMax DIV 2;
    nextColour:=1;

    FOR c1:=1 TO numsnakes DO
      WITH snake[c1] DO
        angle := Random (ScaleAngles);
        angMove := 0;
        angMoveCount := Random (50)+50;
        snakeColour := nextColour + 15;
        nextColour := nextColour MOD (NumColours - 1) + ColourStep;
        xHead := xmid * PointScale;
        yHead := ymid * PointScale;
        FOR c2:=1 TO snakelength DO
          WITH part[c2] DO
            x:=xmid;
            y:=ymid;
          END; (* WITH part *)
        END; (* FOR c2 *)
      END; (* WITH snake *)
    END; (* FOR c1 *)
    xmax := xMax * PointScale;
    ymax := yMax * PointScale;
  END Init;

  PROCEDURE DoDelay (delay:CARDINAL);

  VAR
    count:CARDINAL;

  BEGIN (* DoDelay *)
    FOR count:=1 TO delay DO END;
  END DoDelay;

  PROCEDURE MoveSnakes;

  VAR
    count,head,angValue:CARDINAL;
    x1,y1:INTEGER;

  BEGIN (* MoveSnakes *)
    head:=1;
    LOOP
      FOR count:=1 TO numsnakes DO
        WITH snake [count] DO
          angValue:=angle DIV AngScale;
          xHead:=xHead+cosine[angValue];
          IF (xHead<0) OR (xHead>=xmax) THEN
            xHead:=xHead-cosine[angValue];
            angle:=ScaleAngles DIV 2 - angle; (* Reflect in y-axis *)
            IF angle<0 THEN angle:=angle + ScaleAngles; END;
            angValue:=angle DIV AngScale;
          END; (* IF x... *)
          yHead:=yHead+sine[angValue];
          IF (yHead<0) OR (yHead>=ymax) THEN
            yHead:=yHead-sine[angValue];
            angle:=ScaleAngles - angle; (* Reflect in x-axis *)
          END; (* IF y... *)
          x1:=xHead DIV PointScale;
          y1:=yHead DIV PointScale;
          HiResDot (x1,y1,snakeColour);
          WITH part[head] DO
            HiResDot (x,y,0);
            x:=x1;
            y:=y1;
          END; (* WITH *)
          IF angMoveCount > 0
            THEN DEC (angMoveCount)
            ELSE
              angMoveCount:=Random(50)+50;
              angMove:=Random (AngVariance*2) + ScaleAngles - AngVariance;
              angMove:=angMove MOD ScaleAngles;
            END; (* IF ELSE *)
          angle:=(angle + angMove) MOD ScaleAngles;
        END; (* WITH snake *)
      END; (* FOR count *)
      head:=(head MOD snakelength) + 1;
      Flush;
      DoDelay (delay);
    END; (* LOOP *)
  END MoveSnakes;

  PROCEDURE FindAngles;

  VAR
    c1:CARDINAL;
    eachstep,a:LONGREAL;

  BEGIN (* FindAngles *)
    eachstep:=PI * 2.0 / FLOAT (NumAngles);
    a:=0.0;
    FOR c1:=0 TO NumAngles-1 DO
      sine[c1]:= round (sin (a) * RealPointScale);
      cosine[c1] := round (cos (a) * RealPointScale);
      a:=a+eachstep;
    END; (* FOR *)
(*    FOR c1:=0 TO NumAngles-1 DO
      cosine[c1]:=sine[(NumAngles * 5 DIV 4 - c1) MOD NumAngles];
    END; (* FOR *) *)
  END FindAngles;

BEGIN (* Snakes *)
  FindAngles;
  GetParams;
  Init;
  MoveSnakes;
END Snakes.
